package com.sillycycle.bagleyd.dino;

/*
 * @(#)Dino2D.java
 *
 * Copyright 1994 - 2023  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import javax.swing.JFrame;

/**
 * The <code>Dino2D</code> class does all the 2D graphics for
 * <code>DinoFrame</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/puzzles.html
 */

public class Dino2D extends Canvas {
	private static final long serialVersionUID = 42L;
	static final int FRONT_LOC = 0;
	static final int BACK_LOC = 11;
	static final int COORD2D = 8;
	static final int MAXX = 3;
	static final int MAXY = 4;
	static final int MAXXY = 4; /* (MAX(MAXX,MAXY)) */
	static final int MAX_RECT = 12; /* (MAXX*MAXY) */

	Point[][] triangleUnit = new Point[Dino.MAX_ORIENT][4];
	Point[] letterUnit = new Point[Dino.MAX_ORIENT];
	int[][] triangleListX = new int[Dino.MAX_ORIENT][4];
	int[][] triangleListY = new int[Dino.MAX_ORIENT][4];
	Point[] letterList = new Point[Dino.MAX_ORIENT];
	Point[] offsetList = new Point[Dino.MAX_ORIENT];
	static final int[/*MAX_RECT*/] planeToCube =
		{6, 0, 6, 1, 2, 3, 6, 4, 6, 6, 5, 6};
	static int[/*MAX_FACES*/] cubeToPlane =
		{1, 3, 4, 5, 7, 10};

	Point pos = new Point(0, 0);
	Point puzzleSize, coreSize, puzzleOffset, letterOffset;
	int delta, orientLineLength;
	int viewLength, faceLength, diamondLength;
	int lastX, lastY;

	JFrame      frame = null;
	Dino       dino = null;
	int	 randomPaints = 0;
	boolean     firstPaint = false, resizePaint = false;
	boolean     scanPaint = false, stackPaint = false;
	boolean     framePaint = false, allPiecesPaint = false;
	boolean     selectPaint = false, releasePaint = false;
	boolean     mouseDown = false, vertical = false;

	public Dino2D(JFrame frame, Dino dino) {
		this.frame = frame;
		this.dino = dino;
		firstPaint = true;
	}

	public boolean checkStarted() {
		return dino.checkStarted();
	}

	void drawOrientLine(Graphics g, int rotation,
			int dx, int dy) {
		int x1 = dx, x2 = dx, y1 = dy, y2 = dy;

		switch (rotation) {
		case Dino.TOP:
			y1 -= delta;
			y2 -= (delta + orientLineLength / 2);
			break;
		case Dino.RIGHT:
			x2 += orientLineLength / 2;
			break;
		case Dino.BOTTOM:
			y1 += delta;
			y2 += delta + orientLineLength / 2;
			break;
		case Dino.LEFT:
			x2 -= (orientLineLength / 2);
			break;
		default:
			System.out.println("drawOrientLine: rotation " +
				rotation);
		}
		g.drawLine(x1, y1, x2, y2);
	}

	static void fillPolygon(Graphics g, Color fillColor, Color lineColor,
			int startX, int startY,
			int[] listX, int[] listY, int n) {
		int[] tempListX = new int[n];
		int[] tempListY = new int[n];

		for (int i = 0; i < n; i++) {
			tempListX[i] = listX[i] + startX;
			tempListY[i] = listY[i] + startY;
		}
		g.setColor(fillColor);
		g.fillPolygon(tempListX, tempListY, n);
		g.setColor(lineColor);
		g.drawPolygon(tempListX, tempListY, n);
	}

	void drawTriangle(int face, int position, int offsetPosition) {
		Graphics g = dino.getGraphics();

		if (g == null)
			return;
		try {
			int rotation, l, newSide;
			Point start = new Point(0, 0);
			Color fillColor, lineColor;

			rotation = dino.cubeLoc[face][position].rotation;
			if (vertical || face != Dino.MAX_FACES - 1) {
				start.x = (cubeToPlane[face] % MAXX) *
					viewLength;
				start.y = (cubeToPlane[face] / MAXX) *
					viewLength;
				newSide = position;
			} else {
				start.x = (cubeToPlane[face] / MAXX) *
					viewLength;
				start.y = (cubeToPlane[face] % MAXX) *
					viewLength;
				newSide = (position + Dino.HALF) % Dino.STRT;
				rotation = (rotation + Dino.HALF) % Dino.STRT;
			}
			start.x += puzzleOffset.x + 2 * delta;
			start.y += puzzleOffset.y + 2 * delta;
			/* triangleListX[newSide][0] = offsetList[newSide].x;
			triangleListY[newSide][0] = offsetList[newSide].y; */
			l = dino.cubeLoc[face][position].face;
			if (offsetPosition == 0) {
				if (dino.mono) {
					if (dino.reverse) {
						lineColor = Color.black;
						fillColor = Color.white;
					} else {
						lineColor = Color.white;
						fillColor = Color.black;
					}
				} else {
					fillColor = dino.faceColor[l];
					lineColor = dino.borderColor;
				}
			} else {
				if (dino.mono) {
					if (dino.reverse) {
						lineColor = Color.white;
						fillColor = Color.black;
					} else {
						lineColor = Color.black;
						fillColor = Color.white;
					}
				} else {
					lineColor = dino.faceColor[l];
					fillColor = dino.borderColor;
				}
			}
			fillPolygon(g, fillColor, lineColor,
				start.x, start.y,
				triangleListX[newSide],
				triangleListY[newSide], 3);
			if (dino.mono) {
				String buf;
				int fontSize;

				g.setColor(lineColor);
				fontSize = diamondLength / 6;
				g.setFont(new Font("arial", 0, fontSize));

				buf = Character.toString(dino.faceNames.charAt(l));
				g.drawString(buf,
					start.x + letterList[newSide].x - 2,
					start.y + letterList[newSide].y - 2);
			}
			if (dino.orientChoice) {
				g.setColor(lineColor);
				drawOrientLine(g, rotation + 2 * Dino.MAX_ORIENT,
					start.x + letterList[newSide].x + 1,
					start.y + letterList[newSide].y - 5);
			}
		} finally {
			g.dispose();
		}
	}

	void drawFace(int face) {
		for (int position = 0; position < Dino.MAX_ORIENT; position++)
			drawTriangle(face, position, 0);
	}

	void drawAllPieces() {
		for (int face = 0; face < Dino.MAX_FACES; face++)
			drawFace(face);
	}

	void narrowSelection() {
		// sets dino.currentFace, dino.currentPosition, dino.currentDirection
		if (dino.currentFace == Dino.MAX_FACES - 1 &&
				dino.currentDirection < Dino.MAX_ORIENT && !vertical)
			if (dino.currentDirection < Dino.MAX_ORIENT)
				dino.currentDirection = (dino.currentDirection +
					Dino.HALF) % Dino.MAX_ORIENT;
			else if (dino.currentDirection >= 2 * Dino.MAX_ORIENT)
				dino.currentDirection = 2 * Dino.MAX_ORIENT +
					(dino.currentDirection + Dino.HALF) %
					Dino.MAX_ORIENT;
		/* Remap to row movement */
/*		if (dino.currentPosition != Dino.MAX_ORIENT) {
			if (dino.currentDirection == Dino.CW)
				dino.currentDirection =
					(dino.currentPosition + 1) %
					Dino.MAX_ORIENT;
			else if (dino.currentDirection == Dino.CCW)
				dino.currentDirection =
					(dino.currentPosition + 3) %
					Dino.MAX_ORIENT;
			else if (dino.currentDirection < Dino.MAX_ORIENT &&
					((dino.currentDirection +
					dino.currentPosition) % 2 == 0))
				dino.currentFace = -1;
		} */
	}

	int positionToPiece(int positionX, int positionY) {
		// sets dino.currentPosition
		int faceX, faceY, i, j, face;
		int x = positionX - puzzleOffset.x;
		int y = positionY - puzzleOffset.y;

		faceX = x / viewLength;
		faceY = y / viewLength;
		i = x - faceX * viewLength;
		j = y - faceY * viewLength;
		if (i + j < viewLength && i - j > 0)
			dino.currentPosition = 0;
		else if (i + j > viewLength && i - j > 0)
			dino.currentPosition = 1;
		else if (i + j > viewLength && j - i > 0)
			dino.currentPosition = 2;
		else if (i + j < viewLength && j - i > 0)
			dino.currentPosition = 3;
		else
			return -1;
		if ((faceX != 1 && faceY != 1) ||
				(faceX >= 3 && vertical) ||
				(faceY >= 3 && !vertical))
			return -1;
		face = planeToCube[faceX + faceY * MAXX];
		if (faceX == 3) {
			face = Dino.MAX_FACES - 1;
			dino.currentPosition = (dino.currentPosition +
				Dino.HALF) % Dino.MAX_ORIENT;
		}
		return face;
	}

	void resizePieces() {
		orientLineLength = diamondLength / 3;
		letterOffset = new Point(-2, 4);
		for (int i = 0; i < Dino.MAX_ORIENT; i++) {
			letterList[i] = new Point(0, 0);
			offsetList[i] = new Point(0, 0);
		}
		for (int i = 0; i < Dino.MAX_ORIENT; i++) {
			int dx = 0, dy = 0;
			switch (i) {
			case 0:
				dx = delta;
				break;
			case 1:
				dx = -diamondLength / 2 + 4 * delta;
				dy = delta;
				break;
			case 2:
				dx = -diamondLength / 2 + 3 * delta;
				dy = -diamondLength / 2 + 4 * delta;
				break;
			case 3:
				dy = -diamondLength / 2 + 3 * delta;
				break;
			default:
			}
			for (int j = 0; j <= 3; j++) {
				triangleListX[i][j] = triangleUnit[i][j].x *
					(diamondLength / 2 - 2 * delta) +
					((j > 0) ? triangleListX[i][j - 1] :
					dx);
				triangleListY[i][j] = triangleUnit[i][j].y *
					(diamondLength / 2 - 2 * delta) +
					((j > 0) ? triangleListY[i][j - 1] :
					dy);
			}
		}

		for (int i = 0; i < Dino.MAX_ORIENT; i++) {
			switch (letterUnit[i].x) {
			case 0:
				letterList[i].x = diamondLength / 5 +
					letterOffset.x;
				break;
			case 1:
				letterList[i].x = diamondLength / 2 - 1 +
					letterOffset.x;
				break;
			case 2:
				letterList[i].x = 4 * diamondLength / 5 - 2 +
					letterOffset.x;
				break;
			default:
				break;
			}
			switch (letterUnit[i].y) {
			case 0:
				letterList[i].y = diamondLength / 5 + 2 +
					letterOffset.y;
				break;
			case 1:
				letterList[i].y = diamondLength / 2 - 1 +
					letterOffset.y;
				break;
			case 2:
				letterList[i].y = 4 * diamondLength / 5 - 3 +
					letterOffset.y;
				break;
			default:
				break;
			}

			switch (triangleUnit[i][0].x) {
			case 0:
				offsetList[i].x = delta - 1;
				break;
			case 1:
				offsetList[i].x = diamondLength -
					2 * delta - 2;
				break;
			case 2:
				offsetList[i].x = diamondLength -
					2 * delta - 2;
				break;
			case 3:
				offsetList[i].x = diamondLength - delta - 2;
				break;
			default:
				break;
			}
			switch (triangleUnit[i][0].y) {
			case 0:
				offsetList[i].y = delta - 2;
				break;
			case 1:
				offsetList[i].y = diamondLength -
					2 * delta - 1;
				break;
			case 2:
				offsetList[i].y = diamondLength +
					2 * delta - 2;
				break;
			case 3:
				offsetList[i].y = diamondLength - delta - 1;
				break;
			default:
				break;
			}
		}
	}

	void drawFrame(boolean show) {
		Graphics g = dino.getGraphics();

		if (g == null)
			return;
		try {
			if (show) {
				Point[] coord = new Point[MAXXY + 1];
				Point letters;
				int fontSize = 2 * faceLength / 9;

				for (int i = 0; i <= MAXXY; i++) {
					coord[i] = new Point(
						i * viewLength + puzzleOffset.x,
						i * viewLength + puzzleOffset.y);
				}
				g.setFont(new Font("arial", 0, fontSize));
				g.setColor(dino.currentForeground);
				g.drawLine(coord[1].x, coord[0].y, coord[2].x, coord[0].y);
				g.drawLine(coord[3].x, coord[1].y, coord[3].x, coord[2].y);
				g.drawLine(coord[1].x, coord[3].y, coord[2].x, coord[3].y);
				g.drawLine(coord[0].x, coord[1].y, coord[0].x, coord[2].y);
				letters = new Point(
					(coord[0].x + coord[1].x) / 2 - delta,
					(coord[0].y + coord[1].y) / 2);
				g.setColor(dino.foreground);
				g.drawString("Front",
					letters.x - 5 * fontSize / 4,
					letters.y + 2 * fontSize / 5);
				letters.x = (coord[2].x + coord[3].x) / 2 - delta;
				letters.y = (coord[2].y + coord[3].y) / 2;
				g.drawString("Back",
					letters.x - fontSize,
					letters.y + fontSize / 5);
				g.setColor(dino.currentForeground);
				g.drawLine(coord[1].x, coord[0].y, coord[2].x, coord[0].y);
				if (vertical) {
					g.drawLine(coord[1].x, coord[0].y, coord[1].x, coord[4].y);
					g.drawLine(coord[2].x, coord[0].y, coord[2].x, coord[4].y);
					g.drawLine(coord[0].x, coord[1].y, coord[3].x, coord[1].y);
					g.drawLine(coord[0].x, coord[2].y, coord[3].x, coord[2].y);
					g.drawLine(coord[1].x, coord[4].y, coord[2].x, coord[4].y);
				} else {
					g.drawLine(coord[0].x, coord[1].y, coord[4].x, coord[1].y);
					g.drawLine(coord[0].x, coord[2].y, coord[4].x, coord[2].y);
					g.drawLine(coord[1].x, coord[0].y, coord[1].x, coord[3].y);
					g.drawLine(coord[2].x, coord[0].y, coord[2].x, coord[3].y);
					g.drawLine(coord[4].x, coord[1].y, coord[4].x, coord[2].y);
				}
			} else {
				/*g.setColor(Color.white);
				g.fillRect(0, 0, coreSize.x, offsetY);
				g.fillRect(0, coreSize.y - offsetY - 1,
					coreSize.x, offsetY + 1);
				g.fillRect(0, 0, offsetX, coreSize.y);
				g.fillRect(coreSize.x - offsetX - 1, 0,
					offsetX + 1, coreSize.y);
				g.setColor(background);
				g.fillRect(puzzleOffset.x + 1,
					puzzleOffset.y + 1,
					coreSize.x - 2 - 2 * puzzleOffset.x,
					coreSize.y - 2 - 2 * puzzleOffset.y);*/
				g.setColor(dino.background);
				g.fillRect(0, 0, coreSize.x, coreSize.y);
			}
		} finally {
			g.dispose();
		}
	}

	void resizePuzzle() {
		coreSize = new Point(dino.getSize().width,
			dino.getSize().height);
		delta = 2;

		vertical = (coreSize.y >= coreSize.x);
		if (vertical)
			faceLength = java.lang.Math.min(coreSize.y / MAXY,
				coreSize.x / MAXX);
		else
			faceLength = java.lang.Math.min(coreSize.y / MAXX,
				coreSize.x / MAXY);
		faceLength = java.lang.Math.max(faceLength - delta - 1, 0);
		diamondLength = faceLength - delta;
		viewLength = faceLength + delta;
		if (vertical) {
			puzzleSize = new Point(MAXX * (viewLength - 1) + delta,
				MAXY * (viewLength - 1) + delta);
		} else {
			puzzleSize = new Point(MAXY * (viewLength - 1) + delta,
				MAXX * (viewLength - 1) + delta);
		}
		puzzleOffset = new Point((coreSize.x - puzzleSize.x) / 2,
			(coreSize.y - puzzleSize.y) / 2);
		resizePieces();
	}

	public void initializePuzzle() {
		triangleUnit[0][0] = new Point(2, 0);
		triangleUnit[0][1] = new Point(-1, 1);
		triangleUnit[0][2] = new Point(-1, -1);
		triangleUnit[0][3] = new Point(2, 0);
		triangleUnit[1][0] = new Point(3, 2);
		triangleUnit[1][1] = new Point(-1, -1);
		triangleUnit[1][2] = new Point(1, -1);
		triangleUnit[1][3] = new Point(0, 2);
		triangleUnit[2][0] = new Point(1, 3);
		triangleUnit[2][1] = new Point(1, -1);
		triangleUnit[2][2] = new Point(1, 1);
		triangleUnit[2][3] = new Point(-2, 0);
		triangleUnit[3][0] = new Point(0, 1);
		triangleUnit[3][1] = new Point(1, 1);
		triangleUnit[3][2] = new Point(-1, 1);
		triangleUnit[3][3] = new Point(0, -2);
		letterUnit[0] = new Point(1, 0);
		letterUnit[1] = new Point(2, 1);
		letterUnit[2] = new Point(1, 2);
		letterUnit[3] = new Point(0, 1);
		resizePuzzle();
	}

	void exposePuzzle() {
		drawFrame(false);
		drawFrame(true);
		dino.drawAllPieces();
	}

	void selectPuzzle(boolean control, int x, int y) {
		if ((dino.currentFace = positionToPiece(x, y)) != -1) { //sets currentPosition
			if (control || dino.practiceChoice || !dino.checkSolved()) {
				selectPaint = true;
				dino.repaint();
				return;
			}
			dino.currentFace = -1;
		}
		dino.currentFace = -1;
	}

	void releasePuzzle(boolean shift, boolean control, boolean alt,
			int x, int y) {
		if (dino.currentFace == -1)
			return;
		dino.nextShift = shift;
		dino.nextControl = control;
		dino.nextAlt = alt;
		int tempPosition = dino.currentPosition;
		dino.nextFace = positionToPiece(x, y); // sets currentPosition
		dino.nextPosition = dino.currentPosition;
		dino.currentPosition = tempPosition;
		releasePaint = true;
		dino.repaint();
	}

	@Override
	public void paint(Graphics g) {
		if (firstPaint) {
			dino.initializePuzzle();
			initializePuzzle();
			firstPaint = false;
			resizePaint = true;
		}
		if (!framePaint && !allPiecesPaint &&
				!selectPaint && !releasePaint &&
				!scanPaint && !stackPaint &&
				randomPaints == 0) {
			resizePaint = true;
		}
		if (resizePaint) {
			resizePuzzle();
			exposePuzzle();
			resizePaint = false;
		}
		if (framePaint) {
			drawFrame(true);
			framePaint = false;
		}
		if (allPiecesPaint) {
			drawAllPieces();
			allPiecesPaint = false;
		}
		if (selectPaint) {
			if (dino.currentFace != -1 && dino.currentPosition != -1)
				drawTriangle(dino.currentFace,
					dino.currentPosition, 1);
			selectPaint = false;
		}
		if (releasePaint) {
			if (dino.currentFace != -1) {
				drawTriangle(dino.currentFace,
					dino.currentPosition, 0);
				dino.selectPieces();
				dino.currentFace = -1;
			}
			releasePaint = false;
		}
		if (scanPaint || stackPaint) {
			if (dino.currentControl) {
				dino.moveControlCb(dino.currentFace,
					dino.currentPosition,
					dino.currentDirection,
					dino.currentStyle);
			} else {
				dino.movePieces(dino.currentFace,
					dino.currentPosition,
					dino.currentDirection,
					dino.currentStyle);
				if (stackPaint && dino.checkSolved()) {
					((DinoFrame) frame).callback(
						DinoInterface.ACTION_SOLVED,
						2);
				}
			}
			dino.currentDirection = dino.currentFace = -1;
			scanPaint = false;
			stackPaint = false;
			try {
				Thread.sleep(dino.delay);
			} catch (InterruptedException e) {
				// ignore
			}
		}
		if (randomPaints > 0) {
			dino.randomizingPieces();
		}
	}

	@Override
	public void update(Graphics g) {
		paint(g); /* no erase */
	}

	public void keyPressed(KeyEvent event) {
		int modifiers = event.getModifiersEx();

		switch (event.getKeyCode()) {
		case KeyEvent.VK_DIVIDE:
			dino.movePuzzleInput(dino.lastX, dino.lastY,
				Dino.CCW,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
			break;
		case KeyEvent.VK_HOME:
		case KeyEvent.VK_NUMPAD7:
			dino.movePuzzleInput(dino.lastX, dino.lastY,
				Dino.TL,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
			break;
		case KeyEvent.VK_UP:
		case KeyEvent.VK_KP_UP:
		case KeyEvent.VK_NUMPAD8:
			dino.movePuzzleInput(dino.lastX, dino.lastY,
				Dino.TOP,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
			break;
		case KeyEvent.VK_PAGE_UP:
		case KeyEvent.VK_NUMPAD9:
			dino.movePuzzleInput(dino.lastX, dino.lastY,
				Dino.TR,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
			break;
		case KeyEvent.VK_LEFT:
		case KeyEvent.VK_KP_LEFT:
		case KeyEvent.VK_NUMPAD4:
			dino.movePuzzleInput(dino.lastX, dino.lastY,
				Dino.LEFT,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
			break;
		case KeyEvent.VK_CLEAR:
		case KeyEvent.VK_NUMPAD5:
			dino.movePuzzleInput(dino.lastX, dino.lastY,
				Dino.CW,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
			break;
		case KeyEvent.VK_RIGHT:
		case KeyEvent.VK_KP_RIGHT:
		case KeyEvent.VK_NUMPAD6:
			dino.movePuzzleInput(dino.lastX, dino.lastY,
				Dino.RIGHT,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
			break;
		case KeyEvent.VK_END:
		case KeyEvent.VK_NUMPAD1:
			dino.movePuzzleInput(dino.lastX, dino.lastY,
				Dino.BL,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
			break;
		case KeyEvent.VK_DOWN:
		case KeyEvent.VK_KP_DOWN:
		case KeyEvent.VK_NUMPAD2:
			dino.movePuzzleInput(dino.lastX, dino.lastY,
				Dino.BOTTOM,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
			break;
		case KeyEvent.VK_PAGE_DOWN:
		case KeyEvent.VK_NUMPAD3:
			dino.movePuzzleInput(dino.lastX, dino.lastY,
				Dino.BR,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
			break;
		default:
		}
	}
}
